; setup our Screen Width and Height here for easier tracking
#ScreenWidth  = 640
#ScreenHeight = 480

; Initialize the sprite and keyboard systems and a 640x480, 16-bit screen
If InitSprite() = 0 Or InitKeyboard() = 0 Or OpenScreen(#ScreenWidth,#ScreenHeight,16,"App Title") = 0
   MessageRequester("Error!", "Unable to Initialize Environment", #PB_MessageRequester_Ok)
   End
EndIf

ClearColor = RGB(0,0,0) 

; Use the PNG decoder for image loads.
UsePNGImageDecoder()

; load in the sprite
LoadSprite(0,"ballframe1.png")
LoadSprite(1,"ballframe2.png")

; get the width and height of the sprites
Sprite_Width  = SpriteWidth(0)
Sprite_Height = SpriteHeight(0)

Structure Balls
    X.w           ; track the X position of the ball
    Y.w           ; track the Y position of the ball
    XDir.b        ; track the X direction of the ball (0=left,1=right)
    YDir.b        ; track the Y direction of the ball (0=up,  1=down)
    Speed.b       ; track the speed of the ball
    Frame.b       ; track the frame we're using in the animation
    FrameTimer.b  ; time between frame changes?
    LastChanged.l ; last frame change time?
EndStructure 

; setup a list of balls
NewList Ball.Balls()

; create a bunch of instances of balls
For BCounter = 0 To 50
    If AddElement(Ball()) <> 0
       Ball()\X = Random(#ScreenWidth - Sprite_Width)
       Ball()\Y = Random(#ScreenHeight - Sprite_Height)
       Ball()\XDir = Random(1)
       Ball()\YDir = Random(1)
       Ball()\Speed = Random(4) + 1
       Ball()\Frame = Random(1)
       Ball()\FrameTimer = Random(100) + 30
       Ball()\LastChanged = ElapsedMilliseconds()
    Else
       MessageRequester("Error!", "Unable to allocate memory for new element", #PB_MessageRequester_Ok)
       End
    EndIf
Next

Repeat
   ; clear the screen
   ClearScreen(ClearColor)

   ; get the current time for processing
   Current_Time = ElapsedMilliseconds()
   
   ForEach Ball()
      ; show our sprite (using transparency)
      DisplayTransparentSprite(Ball()\Frame,Ball()\X,Ball()\Y)

      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ; First let's work with the X-Axis
      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ; if we're moving left, subtract speed
      If Ball()\XDir = 0
         Ball()\X = Ball()\X - Ball()\Speed
      Else
         ; we must be moving right, add speed
         Ball()\X = Ball()\X + Ball()\Speed
      EndIf

      ; if we hit the right edge, start moving left
      If Ball()\X > #ScreenWidth - Sprite_Width
         Ball()\XDir = 0
      EndIf
      ; if we hit the left edge, start moving right
      If Ball()\X < 1
         Ball()\XDir = 1
      EndIf
      
      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ; Now let's work with the Y-Axis
      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ; if we're moving up, subtract speed
      If Ball()\YDir = 0
         Ball()\Y = Ball()\Y - Ball()\Speed
      Else
         ; we must be moving down, add speed
         Ball()\Y = Ball()\Y + Ball()\Speed
      EndIf

      ; if we hit the bottom edge, start moving up
      If Ball()\Y > #ScreenHeight - Sprite_Height
         Ball()\YDir = 0
      EndIf
      ; if we hit the top edge, start moving down
      If Ball()\Y < 1
         Ball()\YDir = 1
      EndIf

      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ; Finally let's update the frame
      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ; has sufficient time elapsed to change the frame on this ball?
      If Current_Time > Ball()\LastChanged + Ball()\FrameTimer
         ;if we're on Frame 0, move to 1
         If Ball()\Frame = 0
            Ball()\Frame = 1
         Else
            ; otherwise move to 0
            Ball()\Frame = 0
         EndIf
         ; make sure to reset the LastChanged time!
         Ball()\LastChanged = Current_Time
      EndIf
   Next   

   ; put up text for exiting
   StartDrawing(ScreenOutput())
     DrawText(0,460,"Press any key To quit")
   StopDrawing()

   FlipBuffers()
   
   ExamineKeyboard()
Until KeyboardReleased(#PB_Key_All)

End



; IDE Options = PureBasic 4.61 (Windows - x64)
; CursorPosition = 17