; setup our Screen Width and Height here for easier tracking
#ScreenWidth  = 640
#ScreenHeight = 480

; Initialize the sprite and keyboard systems and a 640x480, 16-bit screen
If InitSprite() = 0 Or InitKeyboard() = 0 Or OpenScreen(#ScreenWidth,#ScreenHeight,16,"App Title") = 0
   MessageRequester("Error!", "Unable to Initialize Environment", #PB_MessageRequester_Ok)
   End
EndIf

ClearColor = RGB(0,0,0) 

; Use the PNG decoder for image loads.
UsePNGImageDecoder()

; load in the sprites
LoadSprite(0,"ball_blue.png")
LoadSprite(1,"ball_red.png")
LoadSprite(2,"ball_green.png")
LoadSprite(3,"ball_yellow.png")

; get the width and height of the sprites
Sprite_Width  = SpriteWidth(0)
Sprite_Height = SpriteHeight(0)

Structure Balls
    Color.b ; 0=blue,1=red,2=green,3=yellow
    X.w     ; track the X position of the ball
    Y.w     ; track the Y position of the ball
    XDir.b  ; track the X direction of the ball (0=left,1=right)
    YDir.b  ; track the Y direction of the ball (0=up,  1=down)
    Speed.b ; track the speed of the ball
EndStructure 

; setup a list of balls
NewList Ball.Balls()

; create a bunch of instances of balls
For BCounter = 0 To 3
    If AddElement(Ball()) <> 0
       Ball()\Color = BCounter
       Ball()\X = Random(#ScreenWidth - Sprite_Width)
       Ball()\Y = Random(#ScreenHeight - Sprite_Height)
       Ball()\XDir = Random(1)
       Ball()\YDir = Random(1)
       Ball()\Speed = 3
    Else
       MessageRequester("Error!", "Unable to allocate memory for new element", #PB_MessageRequester_Ok)
       End
    EndIf
Next

; set up some variable to track hits
Hit.b = 0
Blue.w = 0
Red.w = 0
Green.w = 0
Yellow.w = 0

Repeat
   ; clear the screen
   ClearScreen(ClearColor)

   ForEach Ball()

      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ; First let's work with the X-Axis
      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ; if we're moving left, subtract speed
      If Ball()\XDir = 0
         Ball()\X = Ball()\X - Ball()\Speed
      Else
         ; we must be moving right, add speed
         Ball()\X = Ball()\X + Ball()\Speed
      EndIf

      ; if we hit the right edge, start moving left
      If Ball()\X > #ScreenWidth - Sprite_Width
         Ball()\XDir = 0
      EndIf
      ; if we hit the left edge, start moving right
      If Ball()\X < 1
         Ball()\XDir = 1
      EndIf

      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ; Now let's work with the X-Axis
      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ; if we're moving up, subtract speed
      If Ball()\YDir = 0
         Ball()\Y = Ball()\Y - Ball()\Speed
      Else
         ; we must be moving down, add speed
         Ball()\Y = Ball()\Y + Ball()\Speed
      EndIf

      ; if we hit the bottom edge, start moving up
      If Ball()\Y > #ScreenHeight - Sprite_Height
         Ball()\YDir = 0
      EndIf
      ; if we hit the top edge, start moving down
      If Ball()\Y < 1
         Ball()\YDir = 1
      EndIf

      ; first let's save the current instance of BALL()
      CurrentBall = Ball()
      ; then let's store the color, and X & Y values for the current ball
      CurrentBallC = Ball()\Color
      CurrentBallX = Ball()\X
      CurrentBallY = Ball()\Y
      
      ; run through the list of balls
      ForEach Ball()
         ; make sure we're not on the current ball
         If Ball() <> CurrentBall
            ; do a pixel-perfect collision check
            If SpritePixelCollision(Ball()\Color,Ball()\X,Ball()\Y,CurrentBallC,CurrentBallX,CurrentBallY)
               ; see which ones need adding to, and add to them!
               Select Ball()\Color
                       Case 0
                             Blue = Blue + 1
                       Case 1
                             Red = Red + 1
                       Case 2
                             Green = Green + 1
                       Case 3
                             Yellow = Yellow + 1
               EndSelect
               Select CurrentBallC
                       Case 0
                             Blue = Blue + 1
                       Case 1
                             Red = Red + 1
                       Case 2
                             Green = Green + 1
                       Case 3
                             Yellow = Yellow + 1
               EndSelect
               ; set a flag to say we've had a collision
               Hit = 1
               ; break out of this loop
               Break
            EndIf
         EndIf
      Next
      ChangeCurrentElement(Ball(),CurrentBall)
      
      ; show our sprite (using transparency)
      DisplayTransparentSprite(Ball()\Color,Ball()\X,Ball()\Y)
   Next   

   
   ; put up text for exiting
   StartDrawing(ScreenOutput())
     BackColor(RGB(0,0,0))
     
     FrontColor(RGB(0,0,255))
     DrawText(0,0, "Blue: " + Str(Blue))
     
     FrontColor(RGB(255,0,0))
     DrawText(100,0,"Red: " + Str(Red))
     
     FrontColor(RGB(0,255,0))
     DrawText(200,0,"Green: " + Str(Green))
     
     FrontColor(RGB(255,255,0))
     DrawText(300,0,"Yellow: " + Str(Yellow))
     
     FrontColor(RGB(255,0,255))
     DrawText(0,460,"Press any key To quit")
   StopDrawing()

   FlipBuffers()

   ; if we had a hit, delay to show the hit and then reset everything
   If Hit = 1
      Delay(500)
      ForEach Ball()
          Ball()\X = Random(#ScreenWidth - Sprite_Width)
          Ball()\Y = Random(#ScreenHeight - Sprite_Height)
          Ball()\XDir = Random(1)
          Ball()\YDir = Random(1)
      Next
      ; be sure to reset the hit flag!
      Hit = 0
   EndIf
   
   ExamineKeyboard()
Until KeyboardReleased(#PB_Key_All)

End



; IDE Options = PureBasic 4.61 (Windows - x64)
; CursorPosition = 169
; FirstLine = 156